@@ -12,40 +12,49 @@ module Bundler
       print_gemfile = options.delete(:print_gemfile)
       print_gemspecs = options.delete(:print_gemspecs)
 
-      out = String.new("Environment\n\n")
-      out << "    Bundler   #{Bundler::VERSION}\n"
-      out << "    Rubygems  #{Gem::VERSION}\n"
-      out << "    Ruby      #{ruby_version}"
-      out << "    GEM_HOME  #{ENV["GEM_HOME"]}\n" unless ENV["GEM_HOME"].nil? || ENV["GEM_HOME"].empty?
-      out << "    GEM_PATH  #{ENV["GEM_PATH"]}\n" unless ENV["GEM_PATH"] == ENV["GEM_HOME"]
-      out << "    RVM       #{ENV["rvm_version"]}\n" if ENV["rvm_version"]
-      out << "    Git       #{git_version}\n"
+      out = String.new("## Environment\n\n```\n")
+      out << "Bundler   #{Bundler::VERSION}\n"
+      out << "Rubygems  #{Gem::VERSION}\n"
+      out << "Ruby      #{ruby_version}"
+      out << "GEM_HOME  #{ENV["GEM_HOME"]}\n" unless ENV["GEM_HOME"].nil? || ENV["GEM_HOME"].empty?
+      out << "GEM_PATH  #{ENV["GEM_PATH"]}\n" unless ENV["GEM_PATH"] == ENV["GEM_HOME"]
+      out << "RVM       #{ENV["rvm_version"]}\n" if ENV["rvm_version"]
+      out << "Git       #{git_version}\n"
+      out << "Platform  #{Gem::Platform.local}\n"
+      out << "OpenSSL   #{OpenSSL::OPENSSL_VERSION}\n" if defined?(OpenSSL::OPENSSL_VERSION)
       %w(rubygems-bundler open_gem).each do |name|
         specs = Bundler.rubygems.find_name(name)
-        out << "    #{name} (#{specs.map(&:version).join(",")})\n" unless specs.empty?
+        out << "#{name} (#{specs.map(&:version).join(",")})\n" unless specs.empty?
       end
 
-      out << "\nBundler settings\n\n" unless Bundler.settings.all.empty?
+      out << "```\n"
+
+      out << "\n## Bundler settings\n\n```\n" unless Bundler.settings.all.empty?
       Bundler.settings.all.each do |setting|
-        out << "    " << setting << "\n"
+        out << setting << "\n"
         Bundler.settings.pretty_values_for(setting).each do |line|
-          out << "      " << line << "\n"
+          out << "  " << line << "\n"
         end
       end
+      out << "```\n"
+
+      return out unless SharedHelpers.in_bundle?
 
       if print_gemfile
-        out << "\n#{Bundler.default_gemfile.relative_path_from(SharedHelpers.pwd)}\n\n"
-        out << "    " << read_file(Bundler.default_gemfile).gsub(/\n/, "\n    ") << "\n"
+        out << "\n## Gemfile\n"
+        out << "\n### #{Bundler.default_gemfile.relative_path_from(SharedHelpers.pwd)}\n\n"
+        out << "```ruby\n" << read_file(Bundler.default_gemfile).chomp << "\n```\n"
 
-        out << "\n#{Bundler.default_lockfile.relative_path_from(SharedHelpers.pwd)}\n\n"
-        out << "    " << read_file(Bundler.default_lockfile).gsub(/\n/, "\n    ") << "\n"
+        out << "\n### #{Bundler.default_lockfile.relative_path_from(SharedHelpers.pwd)}\n\n"
+        out << "```\n" << read_file(Bundler.default_lockfile).chomp << "\n```\n"
       end
 
       if print_gemspecs
         dsl = Dsl.new.tap {|d| d.eval_gemfile(Bundler.default_gemfile) }
+        out << "\n## Gemspecs\n" unless dsl.gemspecs.empty?
         dsl.gemspecs.each do |gs|
-          out << "\n#{File.basename(gs.loaded_from)}"
-          out << "\n\n    " << read_file(gs.loaded_from).gsub(/\n/, "\n    ") << "\n"
+          out << "\n### #{File.basename(gs.loaded_from)}"
+          out << "\n\n```ruby\n" << read_file(gs.loaded_from).chomp << "\n```\n"
         end
       end
 
